@@ -90,6 +90,21 @@ def create_settings(nested):
     return type('Settings', (Settings,), {'nested': nested})()
 
 
+class TopLevelSettings(Settings):
+    _host = None
+
+    @property
+    def host(self):
+        if self._host:
+            return self._host
+        else:
+            return default_host(self.license_key)
+
+    @host.setter
+    def host(self, value):
+        self._host = value
+
+
 class AttributesSettings(Settings):
     pass
 
@@ -244,10 +259,31 @@ class InfiniteTracingSettings(Settings):
     @trace_observer_host.setter
     def trace_observer_host(self, value):
         if value and self._can_enable_infinite_tracing():
-            self._trace_observer_host = value
+            self._trace_observer_host = self._normalize_host(value)
         else:
             self._trace_observer_host = None
 
+    @staticmethod
+    def _normalize_host(value):
+        u = urlparse.urlparse(value)
+        if u.hostname:
+            _logger.warning(
+                "An invalid host (%s) was configured for infinite tracing. "
+                "A host of %s has been extracted by interpreting the "
+                "configuration value as a URL. An update to the configuration "
+                "value is recommended.",
+                value, u.hostname)
+            return u.hostname
+        elif "/" in value or ":" in value:
+            _logger.error(
+                "An invalid host (%s) was configured for infinite tracing. "
+                "Please verify that only the host name was supplied via the "
+                "infinite tracing configuration. "
+                "Falling back to infinite tracing disabled.", value)
+            return None
+
+        return value
+
     @staticmethod
     def _can_enable_infinite_tracing():
         if grpc is None:
@@ -282,7 +318,7 @@ class EventHarvestConfigHarvestLimitSettings(Settings):
     nested = True
 
 
-_settings = Settings()
+_settings = TopLevelSettings()
 _settings.attributes = AttributesSettings()
 _settings.thread_profiler = ThreadProfilerSettings()
 _settings.transaction_tracer = TransactionTracerSettings()
@@ -481,8 +517,7 @@ _settings.api_key = os.environ.get('NEW_RELIC_API_KEY', None)
 
 _settings.ssl = _environ_as_bool('NEW_RELIC_SSL', True)
 
-_settings.host = os.environ.get('NEW_RELIC_HOST',
-        default_host(_settings.license_key))
+_settings.host = os.environ.get('NEW_RELIC_HOST')
 _settings.port = int(os.environ.get('NEW_RELIC_PORT', '0'))
 
 _settings.agent_run_id = None
