@@ -1,6 +1,23 @@
-from newrelic.agent import (current_transaction, wrap_object,
-        callable_name, FunctionTrace, ObjectProxy, DatabaseTrace,
-        register_database_client)
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+from newrelic.api.database_trace import DatabaseTrace, register_database_client
+from newrelic.api.function_trace import FunctionTrace
+from newrelic.api.transaction import current_transaction
+from newrelic.common.object_names import callable_name
+from newrelic.common.object_wrapper import wrap_object, ObjectProxy
+from newrelic.core.config import global_settings
 
 DEFAULT = object()
 
@@ -15,13 +32,13 @@ class CursorWrapper(ObjectProxy):
     def execute(self, sql, parameters=DEFAULT, *args, **kwargs):
         transaction = current_transaction()
         if parameters is not DEFAULT:
-            with DatabaseTrace(transaction, sql, self._nr_dbapi2_module,
+            with DatabaseTrace(sql, self._nr_dbapi2_module,
                     self._nr_connect_params, self._nr_cursor_params,
                     parameters, (args, kwargs)):
                 return self.__wrapped__.execute(sql, parameters,
                         *args, **kwargs)
         else:
-            with DatabaseTrace(transaction, sql, self._nr_dbapi2_module,
+            with DatabaseTrace(sql, self._nr_dbapi2_module,
                     self._nr_connect_params, self._nr_cursor_params,
                     None, (args, kwargs)):
                 return self.__wrapped__.execute(sql, **kwargs)
@@ -33,19 +50,19 @@ class CursorWrapper(ObjectProxy):
         except (TypeError, IndexError):
             parameters = DEFAULT
         if parameters is not DEFAULT:
-            with DatabaseTrace(transaction, sql, self._nr_dbapi2_module,
+            with DatabaseTrace(sql, self._nr_dbapi2_module,
                     self._nr_connect_params, self._nr_cursor_params,
                     parameters):
                 return self.__wrapped__.executemany(sql, seq_of_parameters)
         else:
-            with DatabaseTrace(transaction, sql, self._nr_dbapi2_module,
+            with DatabaseTrace(sql, self._nr_dbapi2_module,
                     self._nr_connect_params, self._nr_cursor_params):
                 return self.__wrapped__.executemany(sql, seq_of_parameters)
 
     def callproc(self, procname, parameters=DEFAULT):
         transaction = current_transaction()
-        with DatabaseTrace(transaction, 'CALL %s' % procname,
-                self._nr_dbapi2_module):
+        with DatabaseTrace('CALL %s' % procname,
+                self._nr_dbapi2_module, self._nr_connect_params):
             if parameters is not DEFAULT:
                 return self.__wrapped__.callproc(procname, parameters)
             else:
@@ -67,12 +84,14 @@ class ConnectionWrapper(ObjectProxy):
 
     def commit(self):
         transaction = current_transaction()
-        with DatabaseTrace(transaction, 'COMMIT', self._nr_dbapi2_module):
+        with DatabaseTrace('COMMIT', self._nr_dbapi2_module,
+                self._nr_connect_params):
             return self.__wrapped__.commit()
 
     def rollback(self):
         transaction = current_transaction()
-        with DatabaseTrace(transaction, 'ROLLBACK', self._nr_dbapi2_module):
+        with DatabaseTrace('ROLLBACK', self._nr_dbapi2_module,
+                self._nr_connect_params):
             return self.__wrapped__.rollback()
 
 class ConnectionFactory(ObjectProxy):
@@ -85,8 +104,16 @@ class ConnectionFactory(ObjectProxy):
 
     def __call__(self, *args, **kwargs):
         transaction = current_transaction()
-        with FunctionTrace(transaction, callable_name(self.__wrapped__),
-                terminal=True, rollup='Database/all'):
+
+        settings = global_settings()
+
+        rollup = []
+        rollup.append('Datastore/all')
+        rollup.append('Datastore/%s/all' %
+                self._nr_dbapi2_module._nr_database_product)
+
+        with FunctionTrace(callable_name(self.__wrapped__),
+                terminal=True, rollup=rollup):
             return self.__connection_wrapper__(self.__wrapped__(
                     *args, **kwargs), self._nr_dbapi2_module, (args, kwargs))
 
