@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import sys
 
 import newrelic.packages.six as six
@@ -8,7 +22,8 @@ import newrelic.api.in_function
 import newrelic.api.out_function
 import newrelic.api.pre_function
 from newrelic.api.object_wrapper import callable_name
-from newrelic.api.web_transaction import WSGIApplicationWrapper
+from newrelic.api.wsgi_application import WSGIApplicationWrapper
+from newrelic.api.time_trace import record_exception
 
 def transaction_name_delegate(*args, **kwargs):
     transaction = newrelic.api.transaction.current_transaction()
@@ -23,7 +38,7 @@ def transaction_name_delegate(*args, **kwargs):
 def wrap_handle_exception(self):
     transaction = newrelic.api.transaction.current_transaction()
     if transaction:
-        transaction.record_exception(*sys.exc_info())
+        record_exception()
 
 def template_name(render_obj, name):
     return name
