@@ -41,7 +41,7 @@ from newrelic.common.encoding_utils import (unpack_field, json_encode,
 from newrelic.common.object_names import callable_name
 from newrelic.common.object_wrapper import (transient_function_wrapper,
         function_wrapper, wrap_function_wrapper, ObjectProxy)
-from newrelic.common.system_info import LOCALHOST_EQUIVALENTS
+from newrelic.common.constants import LOCALHOST_EQUIVALENTS
 
 from newrelic.config import initialize
 
@@ -2297,9 +2297,9 @@ def override_ignore_status_codes(status_codes):
     @function_wrapper
     def _override_ignore_status_codes(wrapped, instance, args, kwargs):
         try:
-            # This is a bit horrible as ignore_status_codes is only
-            # used direct from the global settings and not the
-            # application settings. We therefore need to patch the
+            # Ignore_status_codes is only used directly
+            # from the global settings and not the application
+            # settings. We therefore need to patch the
             # global settings directly.
 
             settings = global_settings()
@@ -2323,12 +2323,8 @@ def code_coverage_fixture(source=['newrelic']):
 
         from coverage import coverage
 
-        env_directory = os.environ.get('TOX_ENVDIR', None)
-
-        if env_directory is not None:
-            coverage_directory = os.path.join(env_directory, 'htmlcov')
-        else:
-            coverage_directory = 'htmlcov'
+        coverage_directory = os.environ.get('TOX_ENVDIR', 'htmlcov')
+        coverage_suffix = os.environ.get('TOX_ENV_NAME', None)
 
         def finalize():
             cov.stop()
@@ -2336,7 +2332,7 @@ def code_coverage_fixture(source=['newrelic']):
 
         request.addfinalizer(finalize)
 
-        cov = coverage(source=source, branch=True)
+        cov = coverage(source=source, branch=True, data_suffix=coverage_suffix)
         cov.start()
 
     return _code_coverage_fixture
