@@ -31,6 +31,13 @@ def version2tuple(version_str):
     return tuple(map(_to_int, parts))
 
 def instance_hostname(hostname):
-    if hostname == 'localhost':
+    if hostname == 'localhost' or hostname == "127.0.0.1":
         hostname = socket.gethostname()
     return hostname
+
+def get_open_port():
+    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    s.bind(("", 0))
+    port = s.getsockname()[1]
+    s.close()
+    return port
